@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.52 1999/08/29 17:28:22 nisse Exp $
+ * $Id: channel.c,v 1.56 1999/09/07 20:07:49 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -224,7 +224,10 @@ static void do_exc_finish_channel_handler(struct exception_handler *s,
 	dealloc_channel(self->table, self->channel_number);
 
 	if (self->table->pending_close && !self->table->next_channel)
-	  EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
+	  {
+	    /* FIXME: Send a SSH_DISCONNECT_BY_APPLICATION message? */
+	    EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
+	  }
       }
       break;
     default:
@@ -340,10 +343,13 @@ void dealloc_channel(struct channel_table *table, int i)
     table->next_channel = i;
 }
 
-void register_channel(struct channel_table *table,
-		      UINT32 local_channel_number,
-		      struct ssh_channel *channel)
+void
+register_channel(struct ssh_connection *connection,
+		 UINT32 local_channel_number,
+		 struct ssh_channel *channel)
 {
+  struct channel_table *table = connection->table;
+  
   assert(table->in_use[local_channel_number]);
   assert(!table->channels[local_channel_number]);
   
@@ -352,7 +358,7 @@ void register_channel(struct channel_table *table,
   /* FIXME: Is this the right place to install this exception handler? */
   channel->e = make_exc_finish_channel_handler(table,
 					       local_channel_number,
-					       channel->e);
+					       connection->e);
 }
 
 struct ssh_channel *lookup_channel(struct channel_table *table, UINT32 i)
@@ -375,74 +381,16 @@ static void adjust_rec_window(struct flow_controlled *f, UINT32 written)
   CAST_SUBTYPE(ssh_channel, channel, f);
 
   A_WRITE(channel->write,
-	  prepare_window_adjust
-	  (channel, written),
-	  channel->e);
+	  prepare_window_adjust(channel, written));
 }
 
 void channel_start_receive(struct ssh_channel *channel)
 {
   A_WRITE(channel->write,
 	  prepare_window_adjust
-	  (channel, channel->max_window - channel->rec_window_size),
-	  channel->e);
+	  (channel, channel->max_window - channel->rec_window_size));
 }
 
-#if 0
-/* Process channel-related status codes. Used by the packet handlers,
- * before returning. */
-static int channel_process_status(struct channel_table *table,
-				  int channel,
-				  int status)
-{
-  struct ssh_channel *c = table->channels[channel];
-  
-  while (!LSH_CLOSEDP(status))
-    {
-      if (status & LSH_CHANNEL_CLOSE)
-	{ /* Close the channel now */ 
-	  if (!c->flags & CHANNEL_SENT_CLOSE)
-	    status |= channel_close(c);
-	  break;
-	}
-      
-      if (status & LSH_CHANNEL_READY_SEND)
-	{
-	  status &= ~ LSH_CHANNEL_READY_SEND;
-	  if (c->send_window_size)
-	    status |= CHANNEL_SEND(c);
-	}
-      else if (status & LSH_CHANNEL_READY_REC)
-	{
-	  status &= ~ LSH_CHANNEL_READY_REC;
-	  status |= channel_start_receive(c);
-	}
-      else
-	break;
-    }
-	
-  if (status & LSH_CHANNEL_FINISHED)
-    {
-      /* Clear this bit */
-      status &= ~LSH_CHANNEL_FINISHED;
-
-      if (c->close)
-	status |= CHANNEL_CLOSE(c);
-      
-      dealloc_channel(table, channel);
-    }
-
-  if (status & LSH_CHANNEL_PENDING_CLOSE)
-    table->pending_close = 1;
-  
-  /* If this was the last channel, close connection */
-  if (table->pending_close && !table->next_channel)
-    status |= LSH_CLOSE;
-
-  return status;
-}
-#endif
-
 
 /* Ugly macros to make it a little simpler to free the input packet at
  * the right time. */
@@ -452,9 +400,6 @@ static int channel_process_status(struct channel_table *table,
                     lsh_string_free((s)); \
                     return; } while(0)
 
-#define ERROR(e, msg) \
-  EXCEPTION_RAISE(e, make_protocol_exception \
-    (SSH_DISCONNECT_PROTOCOL_ERROR, msg))
 
 /* Channel related messages */
 
@@ -508,11 +453,10 @@ do_global_request_response(struct global_request_callback *c,
 
       /* FIXME: Perhaps install some exception handler that cancels
        * the queue as soon as a write failes. */
-      A_WRITE(self->super.connection->write,
+      C_WRITE(self->super.connection,
 	      (n->status
 	       ? format_global_success()
-	       : format_global_failure()),
-	      self->super.connection->e);
+	       : format_global_failure()));
     }
 }
 
@@ -575,7 +519,7 @@ static void do_global_request(struct packet_handler *s UNUSED,
     }
   else
     {
-      ERROR(connection->e, "Invalid SSH_MSG_GLOBAL_REQUEST message.");
+      PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_GLOBAL_REQUEST message.");
     }
   lsh_string_free(packet);
 }
@@ -587,7 +531,7 @@ do_global_request_success(struct packet_handler *s UNUSED,
 {
   if (packet->length != 1)
     {
-      ERROR(connection->e, "Invalid GLOBAL_REQUEST_SUCCESS message.");
+      PROTOCOL_ERROR(connection->e, "Invalid GLOBAL_REQUEST_SUCCESS message.");
       RETURN;
     }
 
@@ -616,7 +560,7 @@ do_global_request_failure(struct packet_handler *s UNUSED,
 {
   if (packet->length != 1)
     {
-      ERROR(connection->e, "Invalid GLOBAL_REQUEST_FAILURE message.");
+      PROTOCOL_ERROR(connection->e, "Invalid GLOBAL_REQUEST_FAILURE message.");
       RETURN;
     }
 
@@ -635,81 +579,6 @@ do_global_request_failure(struct packet_handler *s UNUSED,
   END(packet);
 }
 
-#if 0
-/* HERE!!! */
-/* FIXME: Split into a continuation and an exception handler */
-/* Callback given to the CHANNEL_OPEN method */
-static int do_channel_open_response(struct channel_open_callback *c,
-                                    struct ssh_channel *channel,
-                                    UINT32 error, char *error_msg,
-                                    struct lsh_string *args)
-{
-  CAST(channel_open_response, closure, c);
-  
-  int local_channel_number;
-
-  if (!channel)
-    {
-      if (error)
-        C_WRITE(closure->super.connection->write,
-		format_open_failure(closure->remote_channel_number,
-				    error, error_msg, ""));
-      else
-	EXCEPTION_RAISE(closure->super.connection->e,
-			make_protocol_exception())
-      return LSH_FAIL | LSH_DIE;
-    }
-
-  /* FIXME: It would be better to allocate or at least reserve a channel number earlier,
-   * so that we can't fail at this point.
-   */
-  if ( (local_channel_number
-            = register_channel(closure->super.connection->channels,
-			       channel)) < 0)
-    {
-      werror("Could not allocate a channel number for opened channel!\n");
-      return A_WRITE(closure->super.connection->write,
-                     format_open_failure(closure->remote_channel_number,
-                                         SSH_OPEN_RESOURCE_SHORTAGE,
-                                         "Could not allocate a channel number "
-                                         "(shouldn't happen...)", ""));
-    }
-
-  /* FIXME: This copying could just as well be done by the
-   * CHANNEL_OPEN handler? Then we can remove the corresponding fields
-   * from the closure as well. */
-  channel->send_window_size = closure->window_size;
-  channel->send_max_packet = closure->max_packet;
-  channel->channel_number = closure->remote_channel_number;
-
-  /* FIXME: Is the channel->write field really needed? */
-  channel->write = closure->super.connection->write;
-
-  return A_WRITE(closure->super.connection->write,
-                 args
-                 ? format_open_confirmation(channel, local_channel_number,
-                                            "%lfS", args)
-                 : format_open_confirmation(channel, local_channel_number,
-                                            ""));
-}
-
-static struct channel_open_response *
-make_channel_open_response(struct ssh_connection* connection,
-			   UINT32 remote_channel_number,
-			   UINT32 window_size,
-			   UINT32 max_packet)
-{
-  NEW(channel_open_response, closure);
-
-  closure->super.response = do_channel_open_response;
-  closure->super.connection = connection;
-  closure->remote_channel_number = remote_channel_number;
-  closure->window_size = window_size;
-  closure->max_packet = max_packet;
-
-  return closure;
-}
-#endif
 
 /* GABA:
    (class
@@ -742,7 +611,7 @@ do_channel_open_continue(struct command_continuation *c,
   /* FIXME: Is the channel->write field really needed? */
   channel->write = self->connection->write;
 
-  register_channel(self->connection->table,
+  register_channel(self->connection,
 		   self->local_channel_number, channel);
 
   /* FIXME: Doesn't support sending extra arguments with the
@@ -893,7 +762,7 @@ static void do_channel_open(struct packet_handler *c UNUSED,
 	}
     }
   else
-    ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_OPEN message.");
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_OPEN message.");
 
   lsh_string_free(packet);
 }     
@@ -945,7 +814,7 @@ do_channel_request(struct packet_handler *closure UNUSED,
 	}
     }
   else
-    ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_REQUEST message.");
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_REQUEST message.");
   
   lsh_string_free(packet);
 }
@@ -981,7 +850,7 @@ do_window_adjust(struct packet_handler *closure UNUSED,
 	    {
 	      channel->send_window_size += size;
 	      if (channel->send_window_size && channel->send)
-		CHANNEL_SEND(channel);
+		CHANNEL_SEND(channel, connection);
 	    }
 	}
       else
@@ -990,13 +859,13 @@ do_window_adjust(struct packet_handler *closure UNUSED,
 	   * ignored silently? */
 	  werror("SSH_MSG_CHANNEL_WINDOW_ADJUST on nonexistant or closed "
 		 "channel %i\n", channel_number);
-	  ERROR(connection->e, "Unexpected CHANNEL_WINDOW_ADJUST");
+	  PROTOCOL_ERROR(connection->e, "Unexpected CHANNEL_WINDOW_ADJUST");
 	}
     }
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_WINDOW_ADJUST message.");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_WINDOW_ADJUST message.");
     }
 }
 
@@ -1063,7 +932,7 @@ do_channel_data(struct packet_handler *closure UNUSED,
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_DATA message.");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_DATA message.");
     }
 }
 
@@ -1143,7 +1012,7 @@ do_channel_extended_data(struct packet_handler *closure UNUSED,
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_EXTENDED_DATA message.");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_EXTENDED_DATA message.");
     }
 }
 
@@ -1173,10 +1042,8 @@ do_channel_eof(struct packet_handler *closure UNUSED,
 	  if (channel->flags & (CHANNEL_RECEIVED_EOF | CHANNEL_RECEIVED_CLOSE))
 	    {
 	      werror("Receiving EOF on channel on closed channel.\n");
-	      EXCEPTION_RAISE
-		(connection->e,
-		 make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					 "Received EOF on channel on closed channel."));
+	      PROTOCOL_ERROR(connection->e,
+			     "Received EOF on channel on closed channel.");
 	    }
 	  else
 	    {
@@ -1195,13 +1062,13 @@ do_channel_eof(struct packet_handler *closure UNUSED,
 	{
 	  werror("EOF on non-existant channel %i\n",
 		 channel_number);
-	  ERROR(connection->e, "EOF on non-existant channel");
+	  PROTOCOL_ERROR(connection->e, "EOF on non-existant channel");
 	}
     }
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_EOF message");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_EOF message");
     }
 }
 
@@ -1231,7 +1098,7 @@ do_channel_close(struct packet_handler *closure UNUSED,
 	  if (channel->flags & CHANNEL_RECEIVED_CLOSE)
 	    {
 	      werror("Receiving multiple CLOSE on channel.\n");
-	      ERROR(connection->e, "Receiving multiple CLOSE on channel.");
+	      PROTOCOL_ERROR(connection->e, "Receiving multiple CLOSE on channel.");
 	    }
 	  else
 	    {
@@ -1263,13 +1130,13 @@ do_channel_close(struct packet_handler *closure UNUSED,
 	{
 	  werror("CLOSE on non-existant channel %i\n",
 		 channel_number);
-	  ERROR(connection->e, "CLOSE on non-existant channel");
+	  PROTOCOL_ERROR(connection->e, "CLOSE on non-existant channel");
 	}
     }
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_CLOSE message");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_CLOSE message");
     }
 }
 
@@ -1315,13 +1182,13 @@ do_channel_open_confirm(struct packet_handler *closure UNUSED,
 	{
 	  werror("Unexpected SSH_MSG_CHANNEL_OPEN_CONFIRMATION on channel %i\n",
 		 local_channel_number);
-	  ERROR(connection->e, "Unexpected CHANNEL_OPEN_CONFIRMATION.");
+	  PROTOCOL_ERROR(connection->e, "Unexpected CHANNEL_OPEN_CONFIRMATION.");
 	}
     }
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_OPEN_CONFIRMATION message.");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_OPEN_CONFIRMATION message.");
     }
 }
 
@@ -1372,7 +1239,7 @@ do_channel_open_failure(struct packet_handler *closure UNUSED,
     }
   else
     {
-      ERROR(connection->e, "Invalid CHANNEL_OPEN_FAILURE message.");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_OPEN_FAILURE message.");
       lsh_string_free(packet);
     }
 }
@@ -1412,7 +1279,7 @@ do_channel_success(struct packet_handler *closure UNUSED,
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_SUCCESS message");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_SUCCESS message");
     }
 }
 
@@ -1454,7 +1321,7 @@ do_channel_failure(struct packet_handler *closure UNUSED,
   else
     {
       lsh_string_free(packet);
-      ERROR(connection->e, "Invalid CHANNEL_FAILURE message.");
+      PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_FAILURE message.");
     }
 }
 
@@ -1541,19 +1408,6 @@ do_connection_service(struct command *s UNUSED,
   COMMAND_RETURN(c, connection);
 }
 
-#if 0
-struct command *make_connection_service(struct alist *global_requests,
-					struct alist *channel_types)
-{
-  NEW(connection_service, self);
-
-  self->super.call = do_connection_service;
-  self->global_requests = global_requests;
-  self->channel_types = channel_types;
-  
-  return &self->super;
-}
-#endif
 
 struct command
 connection_service = STATIC_COMMAND(do_connection_service);
@@ -1574,7 +1428,7 @@ void channel_close(struct ssh_channel *channel)
   
   channel->flags |= CHANNEL_SENT_CLOSE;
 
-  A_WRITE(channel->write, format_channel_close(channel), channel->e);
+  A_WRITE(channel->write, format_channel_close(channel) );
 
   if (channel->flags & CHANNEL_RECEIVED_CLOSE)
     EXCEPTION_RAISE(channel->e, &finish_exception);
@@ -1592,7 +1446,7 @@ void channel_eof(struct ssh_channel *channel)
   assert(! (channel->flags & CHANNEL_SENT_EOF));
   
   channel->flags |= CHANNEL_SENT_EOF;
-  A_WRITE(channel->write, format_channel_eof(channel), channel->e);
+  A_WRITE(channel->write, format_channel_eof(channel) );
 
   if ( (channel->flags & CHANNEL_CLOSE_AT_EOF)
        && (channel->flags & CHANNEL_RECEIVED_EOF))
@@ -1666,27 +1520,26 @@ struct lsh_string *channel_transmit_extended(struct ssh_channel *channel,
        (type simple UINT32)))
 */
 
-static void do_channel_write(struct abstract_write *w,
-			     struct lsh_string *packet,
-			     struct exception_handler *e)
+static void
+do_channel_write(struct abstract_write *w,
+		 struct lsh_string *packet)
 {
-  struct channel_write *closure = (struct channel_write *) w;
+  CAST(channel_write, closure, w);
   
   A_WRITE(closure->channel->write,
-	  channel_transmit_data(closure->channel, packet), e);
+	  channel_transmit_data(closure->channel, packet) );
 }
 
-static void do_channel_write_extended(struct abstract_write *w,
-				     struct lsh_string *packet,
-				     struct exception_handler *e)
+static void
+do_channel_write_extended(struct abstract_write *w,
+			  struct lsh_string *packet)
 {
   CAST(channel_write_extended, closure, w);
 
   A_WRITE(closure->super.channel->write,
 	  channel_transmit_extended(closure->super.channel,
 				    closure->type,
-				    packet),
-	  e);
+				    packet));
 }
 
 struct abstract_write *make_channel_write(struct ssh_channel *channel)
@@ -1732,7 +1585,8 @@ struct io_read_callback *make_channel_read_stderr(struct ssh_channel *channel)
 */
 
 /* Close callback for files we are writing to. */
-static int channel_close_callback(struct close_callback *c, int reason)
+static void
+channel_close_callback(struct close_callback *c, int reason)
 {
   CAST(channel_close_callback, closure, c);
 
@@ -1753,8 +1607,6 @@ static int channel_close_callback(struct close_callback *c, int reason)
       channel_close(closure->channel);
       break;
     }
-  /* FIXME: So far, the returned value is ignored. */
-  return 17;
 }
 
 struct close_callback *make_channel_close(struct ssh_channel *channel)
@@ -1767,9 +1619,10 @@ struct close_callback *make_channel_close(struct ssh_channel *channel)
   return &closure->super;
 }
 
-struct lsh_string *prepare_channel_open(struct channel_table *table,
-					int type, struct ssh_channel *channel,
-					const char *format, ...)
+struct lsh_string *
+prepare_channel_open(struct ssh_connection *connection,
+		     int type, struct ssh_channel *channel,
+		     const char *format, ...)
 {
   int index;
     
@@ -1788,12 +1641,12 @@ struct lsh_string *prepare_channel_open(struct channel_table *table,
 	channel->rec_max_packet,
 	channel->max_window);
   
-  index = alloc_channel(table);
+  index = alloc_channel(connection->table);
 
   if (index < 0)
     return 0;
 
-  register_channel(table, index, channel);
+  register_channel(connection, index, channel);
 
   l1 = ssh_format_length(OPEN_FORMAT, OPEN_ARGS);
   
