@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.41 1999/08/28 19:57:03 nisse Exp $
+ * $Id: channel.h,v 1.43 1999/09/07 20:07:50 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -104,7 +104,7 @@
        (receive method void "int type" "struct lsh_string *data")
 
        ; Called when we are allowed to send data on the channel. 
-       (send method void)
+       (send method void "struct ssh_connection *connection")
 
        ; Called when the channel is closed
        ; FIXME: Is this needed for anything?
@@ -130,7 +130,7 @@
 #define CHANNEL_RECEIVE(s, t, d) \
 ((s)->receive((s), (t), (d)))
 
-#define CHANNEL_SEND(s) ((s)->send((s)))
+#define CHANNEL_SEND(s, c) ((s)->send((s), (c)))
      
 #define CHANNEL_CLOSE(s) \
 ((s)->close((s)))
@@ -197,7 +197,6 @@
        ))
 */
 
-#define CHANNEL_EXC_HANDLER(c) (&((c)->table.super))
 
 /* SSH_MSG_GLOBAL_REQUEST */
 
@@ -237,25 +236,6 @@
 
 struct exception *make_channel_open_exception(UINT32 error_code, char *msg);
 
-#if 0
-/* Callback function, used to report success or failure for a
- * requested channel open. */
-  
-/* ;;GABA:
-   (class
-     (name channel_open_callback)
-     (vars
-       (response method int
-                "struct ssh_channel *channel"
-                "UINT32 error"
-                "char *error_msg"
-                "struct lsh_string *args")
-       (connection object ssh_connection)))
-*/
-
-#define CHANNEL_OPEN_CALLBACK(c, ch, e, m, a) \
-  (c)->response((c), (ch), (e), (m), (a))
-#endif
 
 /* GABA:
    (class
@@ -302,7 +282,7 @@ struct channel_table *make_channel_table(void);
 int alloc_channel(struct channel_table *table);
 void dealloc_channel(struct channel_table *table, int i);
 
-void register_channel(struct channel_table *table,
+void register_channel(struct ssh_connection *connection,
 		      UINT32 local_channel_number,
 		      struct ssh_channel *channel);
 
@@ -332,7 +312,7 @@ struct lsh_string *prepare_window_adjust(struct ssh_channel *channel,
 
 void channel_start_receive(struct ssh_channel *channel);
 
-struct lsh_string *prepare_channel_open(struct channel_table *table,
+struct lsh_string *prepare_channel_open(struct ssh_connection *connection,
 					int type,
 					struct ssh_channel *channel,
 					const char *format, ...);
@@ -359,9 +339,5 @@ struct lsh_string *channel_transmit_extended(struct ssh_channel *channel,
 
 extern struct command connection_service;
 
-#if 0
-struct command *make_connection_service(struct alist *global_requests,
-					struct alist *channel_types);
-#endif
 
 #endif /* LSH_CHANNEL_H_INCLUDED */
