@@ -1,6 +1,6 @@
 /* channel_commands.c
  *
- * $Id: channel_commands.c,v 1.13 1999/08/28 19:57:04 nisse Exp $
+ * $Id: channel_commands.c,v 1.16 1999/09/07 20:07:51 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -24,6 +24,7 @@
 
 #include "channel_commands.h"
 
+#include "ssh.h"
 #include "werror.h"
 #include "xalloc.h"
 
@@ -49,7 +50,10 @@ void do_channel_open_command(struct command *s,
     {
       /* Probably, we have run out of channel numbers. */
       werror("do_channel_open_command: NEW_CHANNEL failed\n");
-      EXCEPTION_RAISE(e, &dummy_exception);
+      EXCEPTION_RAISE(e,
+		      make_channel_open_exception(
+			SSH_OPEN_RESOURCE_SHORTAGE,
+			"Allocating a local channel number failed."));
     }
   else
     {
@@ -76,7 +80,7 @@ void do_channel_request_command(struct command *s,
     object_queue_add_tail(&channel->pending_requests,
 			  &make_command_context(c, e)->super);
   
-  A_WRITE(channel->write, request, e);
+  A_WRITE(channel->write, request);
 }
 
 void do_channel_global_command(struct command *s,
