@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.71 1999/08/29 17:43:07 nisse Exp $ */
+ * $Id: client.c,v 1.75 1999/09/07 20:07:52 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -81,9 +81,15 @@ do_accept_service(struct packet_handler *c,
   
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_SERVICE_ACCEPT)
-      && parse_atom(&buffer, &name)
-      && parse_eod(&buffer)
-      && (name == closure->service))
+      && (
+#if DATAFELLOWS_WORKAROUNDS
+	  (connection->peer_flags & PEER_SERVICE_ACCEPT_KLUDGE)
+#else
+	  0
+#endif
+	  || (parse_atom(&buffer, &name)
+	      && (name == closure->service)))
+      && parse_eod(&buffer))
     {
       lsh_string_free(packet);
       connection->dispatch[SSH_MSG_SERVICE_ACCEPT] = connection->fail;
@@ -93,9 +99,7 @@ do_accept_service(struct packet_handler *c,
   else
     {
       lsh_string_free(packet);
-      EXCEPTION_RAISE(closure->e,
-		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					      "Invalid SSH_MSG_SERVICE_ACCEPT message"));
+      PROTOCOL_ERROR(closure->e, "Invalid SSH_MSG_SERVICE_ACCEPT message");
     }
 }
 
@@ -248,10 +252,7 @@ do_exit_status(struct channel_request *c,
     }
   else
     /* Invalid request */
-    EXCEPTION_RAISE
-      (channel->e,
-       make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-			       "Invalid exit-status message"));
+    PROTOCOL_ERROR(channel->e, "Invalid exit-status message");
 }
 
 static void
@@ -303,10 +304,7 @@ do_exit_signal(struct channel_request *c,
     }
   else
     /* Invalid request */
-    EXCEPTION_RAISE
-      (channel->e,
-       make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-			       "Invalid exit-signal message"));
+    PROTOCOL_ERROR(channel->e, "Invalid exit-signal message");
 }
 
 struct channel_request *make_handle_exit_status(int *exit_status)
@@ -341,10 +339,10 @@ do_receive(struct ssh_channel *c,
   switch(type)
     {
     case CHANNEL_DATA:
-      A_WRITE(&closure->out->write_buffer->super, data, c->e);
+      A_WRITE(&closure->out->write_buffer->super, data);
       break;
     case CHANNEL_STDERR_DATA:
-      A_WRITE(&closure->err->write_buffer->super, data, c->e);
+      A_WRITE(&closure->err->write_buffer->super, data);
       break;
     default:
       fatal("Internal error!\n");
@@ -352,13 +350,28 @@ do_receive(struct ssh_channel *c,
 }
 
 /* We may send more data */
-static void do_send(struct ssh_channel *c)
+static void
+do_send(struct ssh_channel *s,
+	struct ssh_connection *c UNUSED)
 {
-  CAST(client_session, closure, c);
+  CAST(client_session, self, s);
 
-  assert(closure->in->super.read);
+  assert(self->in->super.read);
 
-  closure->in->super.want_read = 1;
+  self->in->super.want_read = 1;
+}
+
+/* Used for the first callback, to register stdin as a resource. */
+static void
+do_send_first(struct ssh_channel *s,
+	      struct ssh_connection *connection)
+{
+  CAST(client_session, self, s);
+
+  REMEMBER_RESOURCE(connection->resources, &self->in->super.super);
+  s->send = do_send;
+
+  do_send(s, connection);
 }
 
 /* We have a remote shell */
@@ -369,36 +382,27 @@ do_client_io(struct command *s UNUSED,
 	     struct exception_handler *e UNUSED)
 
 {
+  CAST(client_session, session, x);
+  struct ssh_channel *channel = &session->super;
   assert(x);
-#if 0
-  if (!x)
-    {
-      werror("do_client_io: Starting shell failed.\n");
-      return LSH_CHANNEL_CLOSE | LSH_FAIL;
-    }
-  else
-#endif
-    {
-      CAST(client_session, session, x);
-      struct ssh_channel *channel = &session->super;
 
-      channel->receive = do_receive;
+  channel->receive = do_receive;
   
-      session->out->super.close_callback
-	= session->err->super.close_callback = make_channel_close(channel);
+  session->out->super.close_callback
+    = session->err->super.close_callback = make_channel_close(channel);
   
-      session->in->super.read = make_channel_read_data(channel);
-      channel->send = do_send;
+  session->in->super.read = make_channel_read_data(channel);
 
-      ALIST_SET(channel->request_types, ATOM_EXIT_STATUS,
-		make_handle_exit_status(session->exit_status));
-      ALIST_SET(channel->request_types, ATOM_EXIT_SIGNAL,
-		make_handle_exit_signal(session->exit_status));
+  channel->send = do_send_first;
 
-      channel->eof = do_client_session_eof;
+  ALIST_SET(channel->request_types, ATOM_EXIT_STATUS,
+	    make_handle_exit_status(session->exit_status));
+  ALIST_SET(channel->request_types, ATOM_EXIT_SIGNAL,
+	    make_handle_exit_signal(session->exit_status));
+
+  channel->eof = do_client_session_eof;
       
-      COMMAND_RETURN(c, channel);
-    }
+  COMMAND_RETURN(c, channel);
 }
 
 struct command client_io =
@@ -461,7 +465,7 @@ new_session(struct channel_open_command *s,
 
   self->session->write = connection->write;
   
-  *request = prepare_channel_open(connection->table, ATOM_SESSION,
+  *request = prepare_channel_open(connection, ATOM_SESSION,
 				  self->session, "");
   if (!*request)
     return NULL;
