@@ -1,6 +1,6 @@
 /* command.h
  *
- * $Id: command.h,v 1.33 1999/08/28 19:57:05 nisse Exp $ */
+ * $Id: command.h,v 1.35 1999/09/06 22:15:11 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -116,7 +116,28 @@ struct command_context *
 make_command_context(struct command_continuation *c,
 		     struct exception_handler *e);
 
-			    
+/* catch command,
+ *
+ * (catch handler body x)
+ */
+
+/* GABA:
+   (class
+     (name catch_command)
+     (super command_simple)
+     (vars
+       (mask . UINT32)
+       (value . UINT32)))
+*/
+
+struct lsh_object *
+do_catch_simple(struct command_simple *s,
+		struct lsh_object *a);
+
+#define STATIC_CATCH_COMMAND(m, v) \
+{ STATIC_COMMAND_SIMPLE(do_catch_simple), (m), (v) }
+
+
 /* Commands that need to collect some arguments before actually doing
  * anything. */
 
@@ -249,6 +270,9 @@ struct command_continuation *
 make_delay_continuation(struct command *f,
 			struct command_continuation *c);
 
+/* Useful clobal commands */
+#define PROG1 (&command_K.super.super)
+
 
 /* The GABA_* macros are used by automatically generated evaluation code */
 
