@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: debug.c,v 1.27 1999/08/22 21:24:06 nisse Exp $ */
+ * $Id: debug.c,v 1.30 1999/09/07 16:48:17 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -44,23 +44,34 @@
 
 static void
 do_debug(struct abstract_write *w,
-	 struct lsh_string *packet,
-	 struct exception_handler *e)
+	 struct lsh_string *packet)
 {
   CAST(packet_debug, closure, w);
-  
-  debug("DEBUG: received packet %xS\n", packet);
-  
-  A_WRITE(closure->super.next, packet, e);
+
+  if (!packet->length)
+    debug("DEBUG: %z empty packet\n", closure->prefix);
+  else
+    {
+      UINT8 type = packet->data[0];
+      if (type == SSH_MSG_USERAUTH_REQUEST)
+	debug("DEBUG: %z %z *****\n",
+	      closure->prefix, packet_types[type]);
+      else
+	debug("DEBUG: %z %z %xS\n",
+	      closure->prefix, packet_types[type],
+	      packet);
+    }
+  A_WRITE(closure->super.next, packet);
 }
 
 struct abstract_write *
-make_packet_debug(struct abstract_write *continuation, const char *prefix)
+make_packet_debug(struct abstract_write *next,
+		  const char *prefix)
 {
   NEW(packet_debug, closure);
 
   closure->super.super.write = do_debug;
-  closure->super.next = continuation;
+  closure->super.next = next;
   closure->prefix = prefix;
 
   return &closure->super.super;
@@ -110,10 +121,7 @@ do_rec_debug(struct packet_handler *self UNUSED,
 	&& parse_eod(&buffer)))
     {
       lsh_string_free(packet);
-      EXCEPTION_RAISE
-	(connection->e,
-	 make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-				 "Invalid DEBUG message."));
+      PROTOCOL_ERROR(connection->e, "Invalid DEBUG message.");
 				 
     }
   else
