@@ -1,6 +1,6 @@
 /* exception.c
  *
- * $Id: exception.c,v 1.4 1999/08/29 17:47:18 nisse Exp $ */
+ * $Id: exception.c,v 1.7 1999/09/07 16:52:35 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,6 +34,8 @@
 #include "exception.h.x"
 #undef GABA_DEFINE
 
+#include "exception.c.x"
+
 static void
 do_default_handler(struct exception_handler *ignored UNUSED,
 		   const struct exception *e)
@@ -49,7 +51,7 @@ do_default_handler(struct exception_handler *ignored UNUSED,
     }
   else
 #endif
-    fatal("Unhandled exception of type %xi: %z\n", e->type, e->msg);
+    fatal("Unhandled exception of type 0x%xi: %z\n", e->type, e->msg);
 }
 
 struct exception_handler default_exception_handler =
@@ -66,6 +68,54 @@ do_ignore_exception_handler(struct exception_handler *self UNUSED,
 struct exception_handler ignore_exception_handler =
 { STATIC_HEADER, do_ignore_exception_handler };
 
+struct exception_handler *
+make_exception_handler(void (*raise)(struct exception_handler *s,
+				     const struct exception *x),
+		       struct exception_handler *parent)
+{
+  NEW(exception_handler, self);
+  self->raise = raise;
+  self->parent = parent;
+
+  return self;
+}
+
+/* GABA:
+   (class
+     (name report_exception_handler)
+     (super exception_handler)
+     (vars
+       (mask . UINT32)
+       (value . UINT32)
+       (prefix . "const char *")))
+*/
+
+static void
+do_report_exception_handler(struct exception_handler *s,
+			    const struct exception *x)
+{
+  CAST(report_exception_handler, self, s);
+
+  if ( (x->type & self->mask) == self->value)
+    werror("%z exception: %z", self->prefix, x->msg);
+  else
+    EXCEPTION_RAISE(self->super.parent, x);
+}
+
+struct exception_handler *
+make_report_exception_handler(UINT32 mask, UINT32 value,
+			      const char *prefix,
+			      struct exception_handler *parent)
+{
+  NEW(report_exception_handler, self);
+  self->super.raise = do_report_exception_handler;
+  self->super.parent = parent;
+  self->mask = mask;
+  self->value = value;
+  self->prefix = prefix;
+
+  return &self->super;
+}
 
 struct exception *make_simple_exception(UINT32 type, const char *msg)
 {
@@ -76,6 +126,9 @@ struct exception *make_simple_exception(UINT32 type, const char *msg)
   return e;
 }
 
+/* Reason == 0 means disconnect without sending any disconnect
+ * message. */
+
 struct exception *
 make_protocol_exception(UINT32 reason, const char *msg)
 {
@@ -84,7 +137,7 @@ make_protocol_exception(UINT32 reason, const char *msg)
 #define MAX_REASON 11
   const char *messages[MAX_REASON+1] =
   {
-    "", "Host not allowed to connect",
+    NULL, "Host not allowed to connect",
     "Protocol error", "Key exchange failed",
     "Host authentication failed", "MAC error",
     "Compression error", "Service not available",
@@ -92,7 +145,6 @@ make_protocol_exception(UINT32 reason, const char *msg)
     "Connection lost", "By application"
   };
     
-  assert(reason > 0);
   assert(reason <= MAX_REASON);
 
 #undef MAX_REASON
