@@ -1,6 +1,6 @@
 /* exception.h
  *
- * $Id: exception.h,v 1.4 1999/08/28 19:57:09 nisse Exp $ */
+ * $Id: exception.h,v 1.9 1999/09/07 16:52:36 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -68,11 +68,12 @@
 
 /* IO related errors */
 #define EXC_IO 0x2000
-#define EXC_CONNECT 0x2001
-#define EXC_RESOLVE 0x2002
+#define EXC_IO_CONNECT 0x2001
 #define EXC_IO_BLOCKING_WRITE 0x2003
 #define EXC_IO_READ 0x2004
 #define EXC_IO_WRITE 0x2005
+#define EXC_IO_OPEN_WRITE 0x2006
+#define EXC_IO_LISTEN 0x2007
 
 /* Not really errors */
 /* EOF was read */
@@ -82,6 +83,9 @@
 #define EXC_AUTH 0x4000
 #define EXC_USERAUTH 0x4001
 
+/* Implies an exception struct with a reply packet in it. */
+#define EXC_USERAUTH_SPECIAL 0x4002
+
 /* Services */
 #define EXC_SERVICE 0x8000
 #define EXC_GLOBAL_REQUEST 0x8001
@@ -107,10 +111,28 @@
  * as soon as all channels are gone. */
 #define EXC_FINISH_PENDING 0x10004
 
+/* DNS errors */
+#define EXC_RESOLVE 0x20001
+
+/* SEXP parse errors */
+#define EXC_SEXP 0x40000
+#define EXC_SEXP_SYNTAX 0x40001
+#define EXC_SEXP_EOF 0x40002
+
+/* SPKI errors */
+#define EXC_SPKI 0x80000
+/* Unexpected or unsupported expression type */
+#define EXC_SPKI_TYPE 0x80001
+
 extern struct exception_handler default_exception_handler;
 struct exception_handler ignore_exception_handler;
 extern struct exception dummy_exception;
 
+struct exception_handler *
+make_report_exception_handler(UINT32 mask, UINT32 value,
+			      const char *prefix,
+			      struct exception_handler *parent);
+     
 /* ;;GABA:
    (class
      (name exception_frame)
@@ -122,6 +144,12 @@ extern struct exception dummy_exception;
 struct exception *
 make_simple_exception(UINT32 type, const char *msg);
 
+/* Create a simple exception handler, with no internal state */
+struct exception_handler *
+make_exception_handler(void (*raise)(struct exception_handler *s,
+				     const struct exception *x),
+		       struct exception_handler *parent);
+
 /* A protocol exception, that normally terminates the connection */
 /* GABA:
    (class
@@ -141,4 +169,8 @@ make_protocol_exception(UINT32 reason, const char *msg);
 #define STATIC_PROTOCOL_EXCEPTION(reason, msg) \
 { { STATIC_HEADER, EXC_PROTOCOL, (msg) }, (reason) }
 
+#define PROTOCOL_ERROR(e, msg) \
+EXCEPTION_RAISE((e), \
+  make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR, (msg)))
+
 #endif /* LSH_EXCEPTION_H_INCLUDED */
