@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: format.c,v 1.27 1999/04/04 20:22:21 nisse Exp $ */
+ * $Id: format.c,v 1.30 1999/08/28 19:57:10 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -301,7 +301,7 @@ void ssh_vformat_write(const char *f, UINT32 size, UINT8 *buffer, va_list args)
 		  goto end_options;
 		}
 	    }
-end_options:
+	end_options:
 		  
 	  if (literal && decimal)
 	    fatal("Internal error!\n");
@@ -593,3 +593,23 @@ struct lsh_string *make_cstring(struct lsh_string *s, int free)
   return res;
 }
 
+/* FIXME: These functions don't really belong here */
+int lsh_string_cmp(const struct lsh_string *a, const struct lsh_string *b)
+{
+  if (a->length < b->length)
+    return -1;
+  else if (a->length > b->length)
+    return 1;
+  else
+    return memcmp(a->data, b->data, a->length);
+}
+
+int lsh_string_cmp_l(const struct lsh_string *a, UINT32 length, const UINT8 *b)
+{
+  if (a->length < length)
+    return -1;
+  else if (a->length > length)
+    return 1;
+  else
+    return memcmp(a->data, b, length);
+}
