@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.70 1999/08/29 17:51:21 nisse Exp $ */
+ * $Id: io.c,v 1.76 1999/09/07 20:07:56 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -102,7 +102,7 @@ int io_iter(struct io_backend *b)
 		/* FIXME: The value returned from the close callback could be used
 		 * to choose an exit code. */
 		if (fd->close_callback)
-		  (void) CLOSE_CALLBACK(fd->close_callback, fd->close_reason);
+		  CLOSE_CALLBACK(fd->close_callback, fd->close_reason);
 		
 		debug("Closing fd %i.\n", fd->fd);
 		
@@ -112,9 +112,16 @@ int io_iter(struct io_backend *b)
 	    *fd_p = fd->next;
 	    continue;
 	  }
+
 	/* FIXME: nfds should probably include only fd:s that we are
-	 * interested in reading or writing. */
-	nfds++;
+	 * interested in reading or writing. However, that makes the
+	 * mapping from struct pollfd to struct lsh_fd a little more
+	 * difficult. */
+#if 0
+	if (fd->want_read || fd->want_write)
+#endif
+	  nfds++;
+
 	fd_p = &fd->next;
       }
 	
@@ -138,6 +145,10 @@ int io_iter(struct io_backend *b)
       {
 	assert(i < nfds);
 
+#if 0
+	if (! (fd->want_read || fd->want_write))
+	  continue;
+#endif
 	fds[i].fd = fd->fd;
 	fds[i].events = 0;
 
@@ -150,13 +161,16 @@ int io_iter(struct io_backend *b)
 	all_events |= fds[i].events;
       }
     assert(i == nfds);
-
+#if 0
+    assert(all_events);
+#else
     if (!all_events)
       {
 	/* Nothing happens */
 	/* NOTE: There might be some callouts left, but we don't wait */
 	return 0;
       }
+#endif
   }
 
   res = poll(fds, nfds, IDLE_TIME);
@@ -196,11 +210,20 @@ int io_iter(struct io_backend *b)
 	assert(i<nfds);
 
 	debug("io.c: poll for fd %i: events = 0x%xi, revents = 0x%xi.\n",
- 	      fds[i].fd, fds[i].events,fds[i].revents);
+ 	      fds[i].fd, fds[i].events, fds[i].revents);
 	
 	if (!fd->super.alive)
 	  continue;
 
+	if (fds[i].revents & POLLNVAL)
+	  {
+	    werror("io.c: poll request on fd %i, for events of type %xi\n"
+		   "      return POLLNVAL, revents = %xi\n",
+		   fds[i].fd, fds[i].events, fds[i].revents);
+	    kill_fd(fd);
+	    continue;
+	  }
+	
 	if (fds[i].revents & POLLHUP)
 	  {
 	    if (fd->want_read)
@@ -242,61 +265,6 @@ int io_iter(struct io_backend *b)
 }
 
 
-/* ;;GABA:
-   (class
-     (name fd_read)
-     (super abstract_read)
-       (vars
-         (fd object lsh_fd)))
-*/
-
-#if 0
-static int do_read(struct abstract_read **r, UINT32 length, UINT8 *buffer)
-{
-  CAST(fd_read, closure, *r);
-
-  if (!length)
-    {
-      werror("io.c: do_read(): Zero length read was requested.\n");
-      return 0;
-    }
-
-  for (;;)
-    {
-      int res;
-
-      if (! (closure->fd->super.alive && closure->fd->want_read) )
-	return 0;
-      
-      if (! (res = read(closure->fd->fd, buffer, length)) )
-	{
-	  debug("Read EOF on fd %i.\n", closure->fd->fd);
-	  return A_EOF;
-	}
-      if (res > 0)
-	return res;
-
-      switch(errno)
-	{
-	case EINTR:
-	  continue;  /* FIXME: Is it really worth looping here,
-		      * instead of in the select loop? */
-	case EWOULDBLOCK:  /* aka EAGAIN */
-	  return 0;
-	case EPIPE:
-	  werror("io.c: read() returned EPIPE! Treating it as EOF.\n");
-	  return A_EOF;
-	default:
-	  werror("io.c: do_read: read() failed (errno %i), %z\n",
-		 errno, STRERROR(errno));
-	  debug("  fd = %i, buffer = %xi, length = %i\n",
-		closure->fd, buffer, length);
-	  return A_FAIL;
-	}
-    }
-}
-#endif
-
 static void do_buffered_read(struct io_read_callback *s,
 			     struct lsh_fd *fd)
 {
@@ -337,6 +305,14 @@ static void do_buffered_read(struct io_read_callback *s,
 
 	  buffer += done;
 	  left -= done;
+
+	  if (fd->want_read && !self->handler)
+	    {
+	      werror("do_buffered_read: Handler disappeared! Ignoring %i bytes\n",
+		     left);
+	      fd->want_read = 0;
+	      return;
+	    }
 	}
 
       if (left)
@@ -344,8 +320,20 @@ static void do_buffered_read(struct io_read_callback *s,
 		left);
     }
   else
-    EXCEPTION_RAISE(fd->e,
-		    make_io_exception(EXC_IO_EOF, fd, 0, "EOF")) ;
+    {
+#if 0
+      EXCEPTION_RAISE(fd->e,
+		      make_io_exception(EXC_IO_EOF, fd, 0, "EOF")) ;
+#endif
+      /* We have read EOF. Pass available == 0 to the handler */
+      assert(fd->super.alive);
+      assert(fd->read);
+      assert(fd->want_read);
+      assert(self->handler);
+
+      READ_HANDLER(self->handler, 0, NULL);
+    }
+	
 }
 
 struct io_read_callback *
@@ -395,9 +383,10 @@ static void do_consuming_read(struct io_read_callback *c,
       else if (res > 0)
 	{
 	  s->length = res;
-	  A_WRITE(self->consumer, s, fd->e);
+	  A_WRITE(self->consumer, s);
 	}
       else
+	/* FIXME: Perhaps pass NULL to the consumer instead? */
 	EXCEPTION_RAISE(fd->e, make_io_exception(EXC_IO_EOF, fd, 0, "EOF")) ;
     }
 }
@@ -411,103 +400,6 @@ void init_consuming_read(struct io_consuming_read *self,
   self->consumer = consumer;
 }
 			 
-#if 0
-static void read_callback(struct lsh_fd *fd)
-{
-  CAST(io_fd, self, fd);
-  int res;
-  
-  assert(self->read_buffer);
-
-  res = read_buffer_fill_fd(self->read_buffer_fill_fd, fd->fd);
-
-  if (res < 0)
-    switch(errno)
-      {
-      case EINTR:
-	return;
-      case EWOULDBLOCK:
-	werror("io.c: read_callback: Unexpected EWOULDBLOCK\n");
-      case EPIPE:
-	fatal("EPIPE should probably be handled as EOF. Not implemented.\n");
-      default:
-	EXCEPTION_RAISE(fd->e, fd,
-			make_io_exception(EXC_IO_READ),
-			errno, NULL);
-      }
-
-  
-  while (fd->super.alive && fd->want_read)
-    {
-      if (!read_buffer_flush())
-	{
-	  EXCEPTION_RAISE(fd->e, make_io_exception(EXC_IO_EOF, fd, 0, "EOF"));
-	  break;
-	}
-    }
-
-  read_buffer_justify(self->read_buffer);
-
-  assert(!(self->read_buffer->start
-	   && fd->super.alive && fd->want_read));
-}
-
-static void read_callback(struct lsh_fd *fd)
-{
-  CAST(io_fd, self, fd);
-  int res;
-
-  struct fd_read r =
-  { { STACK_HEADER, do_read }, fd->fd };
-
-  /* The handler function may install a new handler */
-  res = READ_HANDLER(self->handler,
-		     &r.super);
-
-  /* NOTE: These flags are not mutually exclusive. All combinations
-   * must be handled correctly. */
-  
-  /* NOTE: (i) If LSH_DIE is set, LSH_CLOSE is ignored. (ii) If the fd
-   * is read_only, LSH_CLOSE is the same as LSH_DIE. */
-
-  /* This condition must be taken care of earlier. */
-  assert(!(res & LSH_CHANNEL_FINISHED));
-
-  /* Not implemented */
-  assert(!(res & LSH_KILL_OTHERS));
-
-  if (res & LSH_HOLD)
-    {
-      /* This flag should not be combined with anything else */
-      assert(res == LSH_HOLD);
-      fd->want_read = 0;
-    }
-  if (res & LSH_DIE)
-    {
-      if (self->buffer)
-	write_buffer_close(self->buffer);
-
-      close_fd(fd,
-	       LSH_FAILUREP(res) ? CLOSE_PROTOCOL_FAILURE : 0);
-    }
-  else if (res & LSH_CLOSE)
-    {
-      if (self->buffer)
-	{
-	  write_buffer_close(self->buffer);
-	  /* Don't attempt to read any further. */
-	  /* FIXME: Is it safe to free the handler here? */
-	  self->super.want_read = 0;
-	  self->handler = NULL;
-	}
-      else
-	kill_fd(fd);
-		  
-      fd->close_reason
-	= LSH_FAILUREP(res) ? CLOSE_PROTOCOL_FAILURE : CLOSE_EOF;
-    }
-}
-#endif
 
 static void write_callback(struct lsh_fd *fd)
 {
@@ -583,7 +475,7 @@ static void connect_callback(struct lsh_fd *fd)
     {
       debug("io.c: connect_callback: Connect failed.\n");
       EXCEPTION_RAISE(fd->e,
-		      make_io_exception(EXC_CONNECT, fd, 0, "connect() failed."));
+		      make_io_exception(EXC_IO_CONNECT, fd, 0, "connect() failed."));
     }
   else
     {
@@ -636,13 +528,39 @@ static void do_kill_fd(struct resource *r)
     close_fd(fd, 0);
 }
 
+/* Closes the file on i/o errors, and passes the exception on */
+
+static void
+do_exc_io_handler(struct exception_handler *self,
+		  const struct exception *x)
+{
+  if (x->type & EXC_IO)
+    {
+      CAST_SUBTYPE(io_exception, e, x);
+
+      switch(x->type)
+	{
+	case EXC_IO_EOF:
+	  close_fd_nicely(e->fd, 0);
+	  break;
+	default:
+	  if (e->fd)
+	    close_fd(e->fd, 0);
+	  break;
+	}
+    }
+  EXCEPTION_RAISE(self->parent, x);
+  return;
+}
+
 /* Initializes a file structure, and adds it to the backend's list. */
 static void init_file(struct io_backend *b, struct lsh_fd *f, int fd,
 		      struct exception_handler *e)
 {
   f->fd = fd;
 
-  f->e = e;
+  f->e = make_exception_handler(do_exc_io_handler, e);
+  
   f->close_reason = -1; /* Invalid reason */
   f->close_callback = NULL;
 
@@ -673,36 +591,11 @@ static void init_file(struct io_backend *b, struct lsh_fd *f, int fd,
  * used to read key-files, so that change probably has to wait until
  * the parser is rewritten. */
 
-#if 0
-/* ;; GABA:
-   (class
-     (name blocking_read_exception_handler)
-     (super exception_handler)
-     (vars
-       (flag . int)))
-*/
-
-static void
-do_blocking_read_handler(struct exception_handler *s,
-			 const struct exception *e UNUSED)
-{
-  CAST(blocking_read_exception_handler, self, s);
-  self->flag = 1;
-}
-#endif
 
 #define BLOCKING_READ_SIZE 4096
 
 int blocking_read(int fd, struct read_handler *handler)
-{
-#if 0
-  struct blocking_read_exception_handler exc_handler =
-  {
-    { STACK_HEADER, do_blocking_read_handler },
-    0
-  };
-#endif
-  
+{  
   char *buffer = alloca(BLOCKING_READ_SIZE);
   
   for (;;)
@@ -965,8 +858,9 @@ void write_raw(int fd, UINT32 length, UINT8 *data,
 	  case EAGAIN:
 	    continue;
 	  default:
-	    EXCEPTION_RAISE(e, make_io_exception(EXC_IO_BLOCKING_WRITE,
-						 NULL, errno, NULL));
+	    EXCEPTION_RAISE(e,
+			    make_io_exception(EXC_IO_BLOCKING_WRITE,
+					      NULL, errno, NULL));
 	    return;
 	  }
       
@@ -1041,10 +935,12 @@ void io_init_fd(int fd)
 }
 
 /* Some code is taken from Thomas Bellman's tcputils. */
-struct connect_fd *io_connect(struct io_backend *b,
-			      struct sockaddr_in *remote,
-			      struct sockaddr_in *local,
-			      struct fd_callback *f)
+struct connect_fd *
+io_connect(struct io_backend *b,
+	   struct sockaddr_in *remote,
+	   struct sockaddr_in *local,
+	   struct fd_callback *f,
+	   struct exception_handler *e)
 {
   int s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
   
@@ -1075,8 +971,7 @@ struct connect_fd *io_connect(struct io_backend *b,
   {
     NEW(connect_fd, fd);
 
-    /* FIXME: What handler to use? */
-    init_file(b, &fd->super, s, &default_exception_handler);
+    init_file(b, &fd->super, s, e);
 
     fd->super.want_write = 1;
     fd->super.write = connect_callback;
@@ -1180,10 +1075,6 @@ struct io_fd *io_read_write(struct io_fd *fd,
   /* Reading */
   fd->super.read = read;
   fd->super.want_read = !!read;
-#if 0
-  fd->handler = handler; 
-  fd->read_buffer = make_fd_read_buffer(READ_BUFFER_SIZE); 
-#endif
   
   /* Writing */
   fd->super.prepare = prepare_write;
@@ -1206,10 +1097,6 @@ struct io_fd *io_read(struct io_fd *fd,
   /* Reading */
   fd->super.want_read = !!read;
   fd->super.read = read;
-#if 0
-  fd->handler = handler;
-  fd->read_buffer = make_fd_read_buffer(READ_BUFFER_SIZE);
-#endif
   
   fd->super.close_callback = close_callback;
 
@@ -1328,3 +1215,17 @@ make_io_exception(UINT32 type, struct lsh_fd *fd, int error, const char *msg)
   
   return &self->super;
 }
+
+struct io_fd *
+io_write_file(struct io_backend *backend,
+	      const char *fname, int flags, int mode,
+	      UINT32 block_size,
+	      struct close_callback *c,
+	      struct exception_handler *e)
+{
+  int fd = open(fname, flags, mode);
+  if (fd < 0)
+    return NULL;
+
+  return io_write(make_io_fd(backend, fd, e), block_size, c);
+}
