@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.41 1999/08/22 21:24:11 nisse Exp $ */
+ * $Id: io.h,v 1.44 1999/09/07 16:55:29 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -76,7 +76,7 @@ struct lsh_fd;
    (class
      (name close_callback)
      (vars
-       (f method int "int reason")))
+       (f method void "int reason")))
 */
 
 #define CLOSE_CALLBACK(c, r) ((c)->f((c), (r)))
@@ -310,10 +310,12 @@ struct exception_handler *
 make_exc_finish_read_handler(struct lsh_fd *fd,
 			     struct exception_handler *parent);
 
-struct connect_fd *io_connect(struct io_backend *b,
-			      struct sockaddr_in *remote,
-			      struct sockaddr_in *local,
-			      struct fd_callback *f);
+struct connect_fd *
+io_connect(struct io_backend *b,
+	   struct sockaddr_in *remote,
+	   struct sockaddr_in *local,
+	   struct fd_callback *f,
+	   struct exception_handler *e);
 
 struct listen_fd *io_listen(struct io_backend *b,
 			    struct sockaddr_in *local,
@@ -341,5 +343,11 @@ void close_fd(struct lsh_fd *fd, int reason);
  * is completely written. */
 void close_fd_nicely(struct lsh_fd *fd, int reason);
 
+struct io_fd *io_write_file(struct io_backend *backend,
+			    const char *fname, int flags,
+			    int mode,
+			    UINT32 block_size,
+			    struct close_callback *c,
+			    struct exception_handler *e);
 
 #endif /* LSH_IO_H_INCLUDED */
