@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.67 1999/08/28 19:57:14 nisse Exp $ */
+ * $Id: lsh.c,v 1.71 1999/09/07 20:07:57 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -37,6 +37,7 @@
 #include "format.h"
 #include "io.h"
 #include "io_commands.h"
+#include "lookup_verifier.h"
 #include "randomness.h"
 #include "service.h"
 #include "ssh.h"
@@ -107,15 +108,18 @@ void usage(void)
        (algorithm object signature_algorithm)))
 */
 
-static struct verifier *do_host_lookup(struct lookup_verifier *c,
-				       struct lsh_string *key)
+static struct verifier *
+do_host_lookup(struct lookup_verifier *c,
+	       struct lsh_string *keyholder UNUSED,	       
+	       struct lsh_string *key)
 {
   CAST(fake_host_db, closure, c);
 
   return MAKE_VERIFIER(closure->algorithm, key->length, key->data);
 }
 
-static struct lookup_verifier *make_fake_host_db(struct signature_algorithm *a)
+static struct lookup_verifier *
+make_fake_host_db(struct signature_algorithm *a)
 {
   NEW(fake_host_db, res);
 
@@ -222,26 +226,35 @@ do_lsh_default_handler(struct exception_handler *s,
 {
   CAST(lsh_default_handler, self, s);
 
-  switch(e->type)
+  if (e->type & EXC_IO)
     {
-    case EXC_CONNECT:
-    case EXC_RESOLVE:
-      werror("lsh: Connection failed: %z\n", e->msg);
-      break;
-    case EXC_IO_READ:
-    case EXC_IO_WRITE:
-      werror("lsh: Connection broken: %z\n", e->msg);
-      break;
-    case EXC_AUTH:
-      werror("lsh: Host authentication failed: %z\n", e->msg);
-      break;
-    case EXC_SERVICE:
-      werror("lsh: Service denied: %z\n", e->msg);
-      break;
-    default:
-      EXCEPTION_RAISE(self->super.parent, e);
+      CAST_SUBTYPE(io_exception, exc, e);
+
+      switch(e->type)
+	{
+	case EXC_IO_EOF:
+	  close_fd_nicely(exc->fd, 0);
+	  break;
+	default:
+	  *self->status = EXIT_FAILURE;
+	  EXCEPTION_RAISE(self->super.parent, e);
+	  return;
+	}
+      werror("lsh: %z, (errno = %i)\n", e->msg, exc->error);
     }
-  *self->status = EXIT_FAILURE;
+  else
+    switch(e->type)
+      {
+      case EXC_RESOLVE:
+      case EXC_AUTH:
+      case EXC_SERVICE:
+	werror("lsh: %z\n", e->msg);
+	*self->status = EXIT_FAILURE;
+	break;
+      default:
+	*self->status = EXIT_FAILURE;
+	EXCEPTION_RAISE(self->super.parent, e);
+      }
 }
 
 static struct exception_handler *
@@ -293,7 +306,7 @@ int main(int argc, char **argv)
   struct keyexchange_algorithm *kex;
   struct alist *algorithms;
   struct make_kexinit *make_kexinit;
-  struct lookup_verifier *lookup;
+  struct alist *lookup_table; /* Alist of signature-algorithm -> lookup_verifier */
 
   struct command *get_pty = NULL;
   
@@ -317,9 +330,15 @@ int main(int argc, char **argv)
   dh = make_dh1(r);
 
   /* No randomness is needed for verifying signatures */
-  lookup = make_fake_host_db(make_dsa_algorithm(NULL)); 
+  lookup_table = make_alist(1
+#if DATAFELLOWS_WORKAROUNDS
+			    +1,
+			    ATOM_SSH_DSS_KLUDGE, make_fake_host_db(make_dsa_kludge_algorithm(NULL))
+#endif
+			    , ATOM_SSH_DSS, make_fake_host_db(make_dsa_algorithm(NULL)),
+			    -1); 
 
-  kex = make_dh_client(dh, lookup);
+  kex = make_dh_client(dh, lookup_table);
   algorithms = many_algorithms(2, 
 			       ATOM_DIFFIE_HELLMAN_GROUP1_SHA1, kex,
 			       ATOM_SSH_DSS, make_dsa_algorithm(r),
