@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: read_data.c,v 1.21 1999/08/22 21:24:19 nisse Exp $ */
+ * $Id: read_data.c,v 1.24 1999/09/07 20:08:02 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -62,45 +62,6 @@ static UINT32 do_read_data_query(struct io_consuming_read *s)
 	     self->channel->send_window_size);
 }
 
-#if 0
-{
-  if (!to_read)
-    {
-      return 
-      /* FIXME: Do this in some other way */
-      /* Stop reading */
-      return LSH_OK | LSH_HOLD;
-    }
-  
-  packet = lsh_string_alloc(to_read);
-  n = A_READ(read, to_read, packet->data);
-
-  switch(n)
-    {
-    case 0:
-      lsh_string_free(packet);
-      return;
-    case A_FAIL:
-      /* Send a channel close, and prepare the channel for closing */
-      lsh_string_free(packet);
-
-      /* FIXME: Raise some appropriate exception */
-      channel_close(closure->channel);
-      return;
-      
-    case A_EOF:
-      if (!--closure->channel->sources)
-	/* Send eof (but no close). */
-	channel_eof(closure->channel);
-      *h = NULL;
-      return;
-    default:
-      packet->length = n;
-
-      A_WRITE(closure->write, packet, e);
-    }
-}
-#endif
 
 struct io_read_callback *make_read_data(struct ssh_channel *channel,
 					struct abstract_write *write)
@@ -118,6 +79,10 @@ struct io_read_callback *make_read_data(struct ssh_channel *channel,
   return &self->super.super;
 }
 
+/* FIXME: This function will be obsoleted with the EXC_IO_EOF
+ * exception type. It would be better to move the handling to the
+ * "consumer" functions in channel.c.*/
+
 /* GABA:
    (class
      (name exc_read_eof_channel_handler)
@@ -126,42 +91,43 @@ struct io_read_callback *make_read_data(struct ssh_channel *channel,
        (channel object ssh_channel)))
 */
 
+/* FIXME: We have to make sure that this handler is invoked exactly
+ * once for each fd attached to the channel. */
 static void
 do_exc_read_eof_channel_handler(struct exception_handler *s,
-				const struct exception *e)
+				const struct exception *x)
 {
   CAST(exc_read_eof_channel_handler, self, s);
 
-  switch(e->type)
+  switch(x->type)
     {
     case EXC_IO_EOF:
       {
-	CAST_SUBTYPE(io_exception, exc, e);
+	CAST_SUBTYPE(io_exception, e, x);
 
 	if (!--self->channel->sources)
 	  /* Send eof (but no close). */
 	  channel_eof(self->channel);
 
-	close_fd_nicely(exc->fd, 0);
+	close_fd_nicely(e->fd, 0);
       }
-	break;
+      break;
     case EXC_IO_READ:
       {
-	CAST_SUBTYPE(io_exception, exc, e);
-	channel_close(self->channel);
+	CAST_SUBTYPE(io_exception, e, x);
 
-	werror("Read error on fd %d (errno = %d): %z\n",
-	       exc->fd->fd, exc->error, e->msg);
+	werror("Read error on fd %i (errno = %i): %z\n",
+	       e->fd->fd, e->error, x->msg);
 
 	if (!--self->channel->sources)
 	  /* Close channel */
 	  channel_close(self->channel);
-	
-	close_fd(exc->fd, 0);
+
+	break;
       }
 	
     default:
-      EXCEPTION_RAISE(self->super.parent, e);
+      EXCEPTION_RAISE(self->super.parent, x);
     }
 }
 
