@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: read_packet.c,v 1.41 1999/08/29 17:55:14 nisse Exp $ */
+ * $Id: read_packet.c,v 1.44 1999/09/07 20:08:03 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -43,25 +43,6 @@
 
 #include "read_packet.c.x"
 
-#if 0
-/* ;;GABA:
-   (class
-     (name flag_exception)
-     (super exception_frame)
-     (vars
-       (flag . int)))
-*/
-
-static void
-do_flag_exception(struct exception_handler *s,
-		  struct exception *e)
-{
-  CAST(flag_exception, self, s);
-  self->flag = 1;
-
-  EXCEPTION_RAISE(self->super.parent, e);
-}
-#endif
 
 /* GABA:
    (class
@@ -113,19 +94,6 @@ lsh_string_realloc(struct lsh_string *s, UINT32 length)
     return s;
 }
 
-#if 0
-
-/* For efficiency, allow reading several packets at a time. But in
- * order not to starve other channels, return when this much data has
- * been read. */
-#define QUANTUM 1024
-
-static const struct io_exception read_exception =
-STATIC_IO_EXCEPTION(EXC_IO, "read_packet: i/o error");
-
-static const struct io_exception read_eof =
-STATIC_IO_EXCEPTION(EXC_IO_EOF, "read_packet: Read EOF");
-#endif
 
 #define READ(n, dst) do {				\
   memcpy((dst)->data + closure->pos, data, (n));	\
@@ -143,8 +111,15 @@ do_read_packet(struct read_handler **h,
   CAST(read_packet, closure, *h);
   UINT32 total = 0;
 
-  assert(available);
-
+  if (!available)
+    {
+      debug("read_packet: EOF in state %i\n", closure->state);
+      EXCEPTION_RAISE(closure->connection->e,
+		      make_protocol_exception(0, "Unexpected EOF"));
+      *h = NULL;
+      return 0;
+    }
+	  
   for (;;)
     switch(closure->state)
       {
@@ -365,7 +340,7 @@ do_read_packet(struct read_handler **h,
 	  closure->packet_buffer = NULL;
 	  closure->state = WAIT_START;
 
-	  A_WRITE(closure->handler, packet, closure->connection->e);
+	  A_WRITE(closure->handler, packet);
 
 	  return total;
 	}
@@ -375,8 +350,9 @@ do_read_packet(struct read_handler **h,
   return total;
 }
 
-struct read_handler *make_read_packet(struct abstract_write *handler,
-				      struct ssh_connection *connection)
+struct read_handler *
+make_read_packet(struct abstract_write *handler,
+		 struct ssh_connection *connection)
 {
   NEW(read_packet, closure);
 
