@@ -1,6 +1,6 @@
 /* server_keyexchange.c
  *
- * $Id: server_keyexchange.c,v 1.28 1999/08/22 21:24:24 nisse Exp $ */
+ * $Id: server_keyexchange.c,v 1.30 1999/09/07 08:57:20 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -104,9 +104,7 @@ do_handle_dh_init(struct packet_handler *c,
       werror("Installing new keys failed. Hanging up.\n");
       KILL(hash);
 
-      EXCEPTION_RAISE(connection->e,
-		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					      "Refusing to use weak key."));
+      PROTOCOL_ERROR(connection->e, "Refusing to use weak key.");
 
       return;
     }
@@ -116,7 +114,10 @@ do_handle_dh_init(struct packet_handler *c,
   connection->kex_state = KEX_STATE_NEWKEYS;
   connection->dispatch[SSH_MSG_KEXDH_INIT] = connection->fail;
 
-  send_verbose(connection, "Key exchange successful!", 0);
+#if DATAFELLOWS_WORKAROUNDS
+  if (! (connection->peer_flags & PEER_SEND_NO_DEBUG))
+#endif
+    send_verbose(connection, "Key exchange successful!", 0);
 
   if (connection->established)
     {
