@@ -136,6 +136,7 @@ int verify_password(struct unix_user *user,
 
 static void
 do_authenticate(struct userauth *ignored UNUSED,
+		struct ssh_connection *connection UNUSED,
 		struct lsh_string *username,
 		struct simple_buffer *args,
 		struct command_continuation *c,
@@ -144,21 +145,11 @@ do_authenticate(struct userauth *ignored UNUSED,
   struct lsh_string *password = NULL;
   /* struct unix_service *service; */
   int change_passwd;
-
-#if 0
-  if (!( (service = ALIST_GET(closure->services, requested_service))))
-    {
-      lsh_string_free(username);
-      return LSH_AUTH_FAILED;
-    }
-#endif
   
   username = utf8_to_local(username, 1, 1);
   if (!username)
     {
-      EXCEPTION_RAISE(e,
-		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					      "Invalid utf8 in password."));
+      PROTOCOL_ERROR(e, "Invalid utf8 in password.");
       return;
     }
 
@@ -185,9 +176,7 @@ do_authenticate(struct userauth *ignored UNUSED,
 	  if (!password)
 	    {
 	      lsh_string_free(username);
-	      EXCEPTION_RAISE(e,
-			      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-						      "Invalid utf8 in password."));
+	      PROTOCOL_ERROR(e, "Invalid utf8 in password.");
 	      return;
 	    }
        
@@ -226,8 +215,7 @@ do_authenticate(struct userauth *ignored UNUSED,
   if (password)
     lsh_string_free(password);
 
-  EXCEPTION_RAISE(e, make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					     "Invalid password USERAUTH message."));
+  PROTOCOL_ERROR(e, "Invalid password USERAUTH message.");
 }
 
 struct userauth unix_userauth =
