@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.13 1999/08/28 19:57:22 nisse Exp $
+ * $Id: server_session.c,v 1.15 1999/09/07 17:08:44 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -26,6 +26,7 @@
 
 #include "channel_commands.h"
 #include "format.h"
+#include "read_data.h"
 #include "server_password.h"
 #include "server_pty.h"
 #include "ssh.h"
@@ -187,7 +188,7 @@ do_receive(struct ssh_channel *c,
   switch(type)
     {
     case CHANNEL_DATA:
-      A_WRITE(&closure->in->write_buffer->super, data, c->e);
+      A_WRITE(&closure->in->write_buffer->super, data);
       break;
     case CHANNEL_STDERR_DATA:
       werror("Ignoring unexpected stderr data.\n");
@@ -200,9 +201,10 @@ do_receive(struct ssh_channel *c,
 
 /* We may send more data */
 static void
-do_send(struct ssh_channel *c)
+do_send(struct ssh_channel *s,
+	struct ssh_connection *c UNUSED)
 {
-  CAST(server_session, session, c);
+  CAST(server_session, session, s);
 
   assert(session->out->super.read);
   /* assert(session->out->handler); */
@@ -301,9 +303,7 @@ do_open_session(struct channel_open *s,
     }
   else
     {
-      EXCEPTION_RAISE(e,
-		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					      "trailing garbage in open message"));
+      PROTOCOL_ERROR(e, "trailing garbage in open message");
     }
 }
 
@@ -450,8 +450,7 @@ static void do_exit_shell(struct exit_callback *c, int signaled,
       A_WRITE(channel->write,
 	      (signaled
 	       ? format_exit_signal(channel, core, value)
-	       : format_exit(channel, value)),
-	      channel->e);
+	       : format_exit(channel, value)) );
 
       if ( (channel->flags & CHANNEL_SENT_EOF)
 	   && (channel->flags & CHANNEL_RECEIVED_EOF))
@@ -569,7 +568,8 @@ static int make_pty(struct pty_info *pty, int *in, int *out, int *err)
 	  "  slave = %i\n"
 	  "... ",
 	  pty->super.alive, pty->master, pty->slave);
-      
+  debug("\n");
+  
   if (pty && pty->super.alive)
     {
       debug("make_pty: Using allocated pty.\n");
@@ -647,9 +647,7 @@ do_spawn_shell(struct channel_request *c,
 
   if (!parse_eod(args))
     {
-      EXCEPTION_RAISE(connection->e,
-		      make_protocol_exception(SSH_DISCONNECT_PROTOCOL_ERROR,
-					      "Invalid shell CHANNEL_REQUEST message."));
+      PROTOCOL_ERROR(connection->e, "Invalid shell CHANNEL_REQUEST message.");
       return;
     }
     
@@ -858,25 +856,35 @@ do_spawn_shell(struct channel_request *c,
 	close(out[1]);
 	close(err[1]);
 
-	/* FIXME: Use proper exception handlers. */
-	session->in
-	  = io_write(make_io_fd(closure->backend, in[1], &default_exception_handler),
-		     SSH_MAX_PACKET,
-		     /* FIXME: Use a proper close callback */
-		     make_channel_close(channel));
-	/* Flow control */
-	session->in->write_buffer->report = &session->super.super;
-	
-	session->out
-	  = io_read(make_io_fd(closure->backend, out[0], &default_exception_handler),
-		    make_channel_read_data(channel),
-		    NULL);
-	session->err 
-	  = ( (err[0] != -1)
-	      ? io_read(make_io_fd(closure->backend, err[0], &default_exception_handler),
-			make_channel_read_stderr(channel),
-			NULL)
-	      : NULL);
+	{
+	  /* Exception handlers */
+	  struct exception_handler *read_exception_handler
+	    = make_exc_read_eof_channel_handler(channel,
+						&default_exception_handler);
+
+	  session->in
+	    = io_write(make_io_fd(closure->backend, in[1],
+				  make_report_exception_handler(
+				    EXC_IO, EXC_IO, "lshd: Child stdin: ",
+				    &default_exception_handler)),
+		       SSH_MAX_PACKET,
+		       /* FIXME: Use a proper close callback */
+		       make_channel_close(channel));
+
+	  /* Flow control */
+	  session->in->write_buffer->report = &session->super.super;
+	  
+	  session->out
+	    = io_read(make_io_fd(closure->backend, out[0], read_exception_handler),
+		      make_channel_read_data(channel),
+		      NULL);
+	  session->err 
+	    = ( (err[0] != -1)
+		? io_read(make_io_fd(closure->backend, err[0], read_exception_handler),
+			  make_channel_read_stderr(channel),
+			  NULL)
+		: NULL);
+	}
 	
 	channel->receive = do_receive;
 	channel->send = do_send;
@@ -901,9 +909,7 @@ do_spawn_shell(struct channel_request *c,
 
 	if (want_reply)
 	  A_WRITE(channel->write,
-		  format_channel_success(channel
-					 ->channel_number),
-		  channel->e);
+		  format_channel_success(channel->channel_number) );
 
 	channel_start_receive(channel);
 	return;
@@ -917,8 +923,8 @@ do_spawn_shell(struct channel_request *c,
   }
  fail:
   if (want_reply)
-    A_WRITE(channel->write, format_channel_failure(channel->channel_number),
-	    channel->e);
+    A_WRITE(channel->write,
+	    format_channel_failure(channel->channel_number));
 }
 
 struct channel_request *make_shell_handler(struct io_backend *backend,
@@ -992,8 +998,7 @@ do_alloc_pty(struct channel_request *c UNUSED,
 		  verbose(" granted.\n");
 		  if (want_reply)
 		    A_WRITE(channel->write,
-			    format_channel_success(channel->channel_number),
-			    channel->e);
+			    format_channel_success(channel->channel_number) );
 		  return;
 		}
 	      else
@@ -1008,8 +1013,8 @@ do_alloc_pty(struct channel_request *c UNUSED,
   lsh_string_free(term);
 
   if (want_reply)
-    A_WRITE(channel->write, format_channel_failure(channel->channel_number),
-	    channel->e);
+    A_WRITE(channel->write,
+	    format_channel_failure(channel->channel_number) );
 }
 
 struct channel_request *make_pty_handler(void)
